;+
; AC les 26 et 27 Septembre 2005
;
; Ce programme ajoute des couleurs a la table standard.
; Par la technique choisie, a priori, on peut conserver
; les couleurs ecrites en dures quelque soit la table chargee
; (en fait, il suffit de repasser ici ...)
;
; Comment ajouter de nouvelles couleurs ?
; 
; On pourra s'aider de "colordemo" sous X11 pour choisir
; des couleurs nommees et normalisees sous X11
;
; ---------------------------------
; Petit rappel historique:  passge de "!col_X" en "!col.X"
;
; Attention, on choisit de changer la structure:
; avant, on avait : !col_r, !col_b
; desormais, on sort une liste !col accompagnee d'une liste de noms
; (!col.name) donc on a : !col.r, !col.b, ...
;
; Lors du 1er appel, table_number permet de changer la table de
; couleur, tout en gardant certaines de nos couleurs
; predefinies "supplemetaires"
;
; --------------------------------------
; Liste des modifications:
; AC le 11/10/05:
; - le nom-clef /Debug permet d'ajouter facilement une couleur ...
;
;-
pro AC_COLORS_EXTRA, table_number=table_number, $
                     help=help, test=test, debug=debug, verbose=verbose
;
common colors, r_orig, g_orig, b_orig, r_curr, g_curr, b_curr
;
mess=AC_SETMESSAGES('AC_COLORS_EXTRA')
;
if KEYWORD_SET(help) then begin
    print, 'pro AC_COLORS_EXTRA, help=help, test=test, verbose=verbose'
    print, ''
    print, 'Purpose: this procedure initializes a custom colors palette'
    print, 'and creates a ReadOnly structure !Col containing colors alias'
    print, 'NB: the /DEBUG keyword helps for expanding color list (if new col)'
    return
endif
;
DEFSYSV, '!col', exists=flag_col
if (flag_col EQ 1) then begin
    print, mess.warning+'The !col Structure was previously created'
    old_table_number=!col.table_number
endif
;
if N_ELEMENTS(table_number) GT 0 then begin
    LOADCT, table_number
endif
;
; Initialisation de la liste du nom des couleurs et de celle des raccourcis
; Parmi les 256 couleurs possibles, certaines ne sont pas
; explicitement nommees. On pourra jouer sur ce champ !
;
name=REPLICATE('Not Allocated',256)
short=REPLICATE('',256)
;
; indice variable
jj=0
;
; Ajout de couleurs d'apres les noms indiques par "colordemo" 
;
; ----------------------- les 8 couleurs de base -------------
; Black, White; Red, Green, Blue; Cyan, Magenta, Yellow
;
; Black (#000000)
ii=jj & jj=jj+1 & r_curr[ii]=0 & g_curr[ii]=0 & b_curr[ii]=0
short(ii)='!col.d' & name(ii)='Black'
;
; White (#FFFFFF)
ii=jj & jj=jj+1 & r_curr[ii]=255 & g_curr[ii]=255 & b_curr[ii]=255
short(ii)='!col.w' & name(ii)='White'
;
; Red (#FF0000)
ii=jj & jj=jj+1 & r_curr[ii]=255 & g_curr[ii]=0 & b_curr[ii]=0
short(ii)='!col.r' & name(ii)='Red'
;
; Blue (#0000FF)
ii=jj & jj=jj+1 & r_curr[ii]=0 & g_curr[ii]=0 & b_curr[ii]=255
short(ii)='!col.b' & name(ii)='Blue'
;
; Green (#0000FF)
ii=jj & jj=jj+1 & r_curr[ii]=0 & g_curr[ii]=255 & b_curr[ii]=0
short(ii)='!col.g' & name(ii)='Green'
;
; Cyan (#00FFFF)
ii=jj & jj=jj+1 & r_curr[ii]=0 & g_curr[ii]=255 & b_curr[ii]=255
short(ii)='!col.c' & name(ii)='Cyan'
;
; Magenta (#FF00FF)
ii=jj & jj=jj+1 & r_curr[ii]=255 & g_curr[ii]=0 & b_curr[ii]=255
short(ii)='!col.m' & name(ii)='Magenta'
;
; Yellow (#FFFF00)
ii=jj & jj=jj+1 & r_curr[ii]=255 & g_curr[ii]=255 & b_curr[ii]=0
short(ii)='!col.y' & name(ii)='Yellow'
;
; ---------------- Quelques Gris ---------------------------
;
; Light Grey86 (#DBDBDB) 
ii=jj & jj=jj+1 & r_curr[ii]=219 & g_curr[ii]=219 & b_curr[ii]=219
short(ii)='!col.lgr' & name(ii)='Light Grey86'
;
; Dark Grey29 (#747474)
ii=jj & jj=jj+1 & r_curr[ii]=74 & g_curr[ii]=74 & b_curr[ii]=74
short(ii)='!col.dgr' & name(ii)='Dark Grey29'
;
; ---------------- Quelques Verts ---------------------------
;
; Light Green (#90EE90)
ii=jj & jj=jj+1 & r_curr[ii]=144 & g_curr[ii]=238 & b_curr[ii]=144
short(ii)='!col.lg' & name(ii)='Light Green'
;
; Dark Green (#006400)
ii=jj & jj=jj+1 & r_curr[ii]=0 & g_curr[ii]=100 & b_curr[ii]=0
short(ii)='!col.dg' & name(ii)='Dark Green'
;
; ---------------- Quelques Bleus ---------------------------
;
; Cadet Blue 1 (#98F5FF)
ii=jj & jj=jj+1 & r_curr[ii]=152 & g_curr[ii]=255 & b_curr[ii]=245
short(ii)='!col.cb' & name(ii)='Cadet Blue'
;
; Dark Blue (#00008B)
ii=jj & jj=jj+1 & r_curr[ii]=0 & g_curr[ii]=0 & b_curr[ii]=139
short(ii)='!col.db' & name(ii)='Dark Blue'
;
; ---------------- Quelques Pink/Violet ---------------------------
; Pink (#FFC0CB)
ii=jj & jj=jj+1 & r_curr[ii]=255 & g_curr[ii]=192 & b_curr[ii]=203
short(ii)='!col.p' & name(ii)='Pink'
;
; Violet Purple (#A020F0)
ii=jj & jj=jj+1 & r_curr[ii]=160 & g_curr[ii]=32 & b_curr[ii]=240
short(ii)='!col.pv' & name(ii)='Violet Purple'
;
; Dark Violet (#9400D3)
ii=jj & jj=jj+1  & r_curr[ii]=148 & g_curr[ii]=0 & b_curr[ii]=211
short(ii)='!col.dv' & name(ii)='Dark Violet'
;
; ---------------------- Autres couleurs ------------------
; Brown 4 (#8B2323)
ii=jj & jj=jj+1 & r_curr[ii]=139 & g_curr[ii]=35 & b_curr[ii]=35
short(ii)='!col.br' & name(ii)='Brown'
;
; Orange (#FFA500)
ii=jj & jj=jj+1 & r_curr[ii]=255 & g_curr[ii]=165 & b_curr[ii]=0
short(ii)='!col.o' & name(ii)='Orange'
;
; --------------- Creation de la structure !col elle-meme ------------
; (je n'ai pas trouve de moyen elegant pour la creer au fur et a mesure)
;
indices_colors=WHERE(name NE 'Not Allocated', nb_colors)
;
; En cas d'extension (ajout de couleurs), il suffit de faire
; un copier-coller de ce qui suit pour agrandir/modifier la structure.
;
if KEYWORD_SET(debug) then begin
    for ii=0, nb_colors-1 do begin
        txt=StrMid(short(indices_colors(ii)), 5)+':'+String(ii)+',$'
        print, StrCompress(txt)
    endfor
endif
;
; creation effective de la structure !col
; qui contient plusieurs champs:
; -- les noms abreges (!col.v, !col.r, ...).
; -- la liste des noms longs, les indices non utilises etant a "Not
;    Allocated" !col.name (256 elements)
; -- la liste des raccourcis !col.short (une dizaine d'elements)
; -- la liste des indices des couleurs initialisees (une dizaine d'elements)
;
color_struct={d: 0,$
              w: 1,$
              r: 2,$
              b: 3,$
              g: 4,$
              c: 5,$
              m: 6,$
              y: 7,$
              lgr: 8,$
              dgr: 9,$
              lg: 10,$
              dg: 11,$
              cb: 12,$
              db: 13,$
              p: 14,$
              pv: 15,$
              dv: 16,$
              br: 17,$
              o: 18,$
              name: name, $
              nb_colors: nb_colors, $
              short: short, $
              indices: indices_colors, $
              table_number: table_number }
;
; si la structure n'existe pas deja, on la cree
;
if (flag_col NE 1) then begin
    DEFSYSV, '!col', color_struct
endif else begin
    print, 'ici, ca existait deja'
    print, !col.nb_colors, color_struct.nb_colors
    print, !col.table_number, color_struct.table_number
endelse
;
if KEYWORD_SET(verbose) then begin
    indices_colors=where(!col.name NE 'Not Allocated', count)
    print, count, ' predefined colors'
    for i=0, count-1 do begin
        print, !col.name(indices_colors(i)), indices_colors(i)
    endfor
endif
;
r_orig=r_curr
b_orig=b_curr
g_orig=g_curr
;
; il faut cette commande pour reellement affecter les nouvelles couleurs
;
TVLCT, r_curr, g_curr, b_curr
;
if KEYWORD_SET(test) then stop
;
end
