;+
; AC le 14/09/2005 a Paris
;
; Ce programme encapsule la procedure intrinseque PLOT
; afin de pouvoir automatiquement recalculer l'echelle
; (de l'ordre d'autour la valeur mediane +- 3 sigma)
; afin de ne avoir a indiquer cela en parametres
;
; Attention, le mot clef qui pourrait permettre de choisir le type
; de STDDEV n'est pas actif.
;
; ------------------------------
;-
pro ACPLOT, x, y, type_of_stddev=type_of_stddev, _extra=_extra, $
            verbose=verbose, help=help
;
if KEYWORD_SET(help) then begin
    print, 'pro ACPLOT, x, y, type_of_stddev=type_of_stddev, _extra=_extra, $'
    print, '            verbose=verbose, help=help'
    print, ' '
    print, 'CALLING SEQUENCE: acplot, x, [y], linestyle=2, /vebose'
    return
endif
;
mess=AC_SETMESSAGES('ACPLOT')
;
if NOT(KEYWORD_SET(x)) then begin
    print, mess.fatal+'missing array'
    return
endif
;
mode=0
if KEYWORD_SET(y) then begin
    med=MEDIAN(y)
    stdref=STDDEV(y)
    std_ac=AC_STDDEV(y)
    mode=1
endif else begin
    med=MEDIAN(x)
    stdref=STDDEV(x)
    std_ac=AC_STDDEV(x)
endelse
;
if (stdref GT 1.5*std_ac) then begin
    print, mess.info+'STDDEV and AC_STDDEV are significantly differents'
    print, mess.info+'Please select TYPE_OF_STDDEV keyword (0|1)'
    print, mess.info+'Default is standard STDDEV'
endif
;
ymin=med-3.*stdref
ymax=med+3.*stdref
yrange=[ymin,ymax]
;
if mode EQ 0 then plot, x, yrange=yrange, _extra=_extra
if mode EQ 1 then plot, x, y, yrange=yrange, _extra=_extra
;
if KEYWORD_SET(verbose) then begin
    print, mess.info+'Number of points: ............. : ', N_ELEMENTS(x)
    print, mess.info+'Standard Deviation ............ : ', stdref
    print, mess.info+'Renormalized Standard Deviation : ', std_ac
    print, mess.info+'Median : ...................... : ', med
endif
;
end
    
