;
; AC le 08/11/2005
; ---------------------------------------
;
; Exemples:
;
; 1/ Dent de scie : On doit voir une Raie principale a 16 Hz !
;
; IDL> duration=1.25
; IDL> sf16=SAW_FISH(freq=16., nbp=512*8, dur=duration)
; IDL> PLOT_POWER_SPECTRUM, sf16, dur=duration, freq_max=36.
; IDL> sf15=SAW_FISH(freq=15.1, nbp=512*8, dur=duration)
; IDL> PLOT_POWER_SPECTRUM, sf15, dur=duration, /over, col=!col.r
; ---------------------------------------
;
; Liste des modifications:
; AC le 17/11/2005: 
;  - ajout affichage frequence pure (central_freq)
;  - ajout doc et aide en ligne (/HELP actif)
;
; ---------------------------------------
;
pro PLOT_POWER_SPECTRUM, signal, duration=duration, $
                         freq_max=freq_max, central_freq=central_freq, $
                         nozero=nozero, verbose=verbose, overplot=overplot, $
                         help=help, test=test, _extra=_extra
;
mess=AC_SETMESSAGES('PLOT_POWER_SPECTRUM')
;
if N_PARAMS() LT 1 then begin
    print, mess.fatal+'missing input SIGNAL field'
    help=1
endif
;
if KEYWORD_SET(help) then begin
    print, 'pro PLOT_POWER_SPECTRUM, signal, duration=duration, $'
    print, '             freq_max=freq_max, central_freq=central_freq, $'
    print, '             nozero=nozero, verbose=verbose, overplot=overplot, $'
    print, '             help=help, test=test, _extra=_extra'
    return
endif
;
; si aucune information est fournie sur le signal,
; il dure une seconde
;
if N_ELEMENTS(duration) EQ 0 then duration=1.  ; En secondes
;
; abscisses
;
nbp=N_ELEMENTS(signal)
freqs=FINDGEN(nbp)/duration
;
pws=AC_POWER_SPECTRUM(signal, nozero=nozero)
;
if N_ELEMENTS(freq_max) EQ 0 then freq_max=MAX(freqs)
;
if NOT(KEYWORD_SET(overplot)) then begin
    plot, freqs, pws, $
      xtitle='Frequences (Hertz)', xrange=[0,freq_max],/xstyle, _extra=_extra
endif else begin
    oplot, freqs, pws, _extra=_extra
endelse
;
; affichage de la frequence pure "principale"
; on ne gere pas si elle est negative !
;
if KEYWORD_SET(central_freq) then begin
    AC_LINES, central_freq, /vertical, color=!col.g
endif
;
end

