;
; AC, le 01/04/2005 a Paris
;
; Les parametres Bolo sont stockes dans un fichier ASCII
; Il faut les relire pour les mettre dans une structure.
;
; Malheureusement, la procedure "READ_ASCII" ne sait pas lire
; les fichiers melangeant des nombres et du texte
; Mais on s'en sert pour recuperer le nombre de lignes utiles
;
;----------------------------------------------
; Conventions pour les fichiers de donnees Bolo
;
; Les lignes de commentaires commencent par un "#"
; Les commentaires sont au debut
; La premiere ligne utile est factice, elle sert pour les relectures
; par tableur (OOcalc, MS-Excel) au format CSV
; Les lignes utiles comptent exactement 10 ";"
; Les lignes utiles peuvent contenir, a la fin, un commentaire prefixe
; par un caractere "#"
;----------------------------------------------
; Liste de modifications:
;
; AC le 03/06/2005 
; -correction d'un bug lie aux versions d'IDL lorsqu'on lit des
; structures anonymes
; (pb trouve grace a CREATE_HTML_GALLERY.pro et READF_CSV.pro)
;
; AC le 02/01/2006
; - chgt de nom de LIST_BOLO_READFILE en READ_BOLO_CSV
; - refonte totale pour un nouveau type defichier CSV
;
;----------------------------------------------
;
function READ_BOLO_CSV, filename=filename, path=path, $
                        prefixe=prefixe, suffixe=suffixe, $
                        default=default, flag=flag, $
                        verbose=verbose, debug=debug, test=test, help=help
;
mess=AC_SETMESSAGES('LIST_BOLO_READFILE')
;
; si le mot-clef /help est active, on retourne l'aide et on sort
;
if KEYWORD_SET(help) then begin
    print, 'function LIST_BOLO_READFILE, filename=filename, path=path, $'
    print, '            verbose=verbose,debug=debug, test=test, help=help'
    print, ''
    print, 'This function reads an ASCII file containing'
    print, 'a list of Bolometers parameters'
    print, 'The output is stored in a list of structure'
    print, ''
    print, 'CALLING SEQUENCE : list_bolo_struct=LIST_BOLO_READFILE()'
    return, -1
endif
;
if N_ELEMENTS(prefixe) EQ 0 then prefixe='boloPFM_'
if N_ELEMENTS(suffixe) EQ 0 then suffixe='.csv'
;
if KEYWORD_SET(default) then begin
    filename='boloPFM_Dark.csv'
    print, mess.info+'Default Bolo FileName used:', filename
endif
;
; AC 21/05/2005 : we do not manage fully the PATH now
;
if N_ELEMENTS(path) EQ 0 then begin
    path='BoloData'+!path_sep
    print, mess.info+'Default Path: ./', path
endif
;
; On s'assure que le Fichier est bien present
;
OK_file=AC_SELECTFILE(filename=filename, path=path, $
                         prefixe=prefixe, suffixe=suffixe)
if (OK_file NE 1) then begin
    print, mess.fatal+'Bolo INFO FileName <<'+filename+'>>,'
    print, mess.fatal+'in PATH <<'+path+'>> does not exist,'
    print, mess.fatal+'Please re-run this function with another filename.'
    return, -1    
endif else begin
    full_filename=path+filename
endelse
;
; Conventions dans le fichier ASCII :
;   - les separateurs sont des ","
;   - les commentaires sont apres des "#"
;   - la premiere colonne contient le Nom du bolo
;   - la 2ieme colonne contient des info sur les unites (K, W, ...)
;
a=READ_ASCII_FILE(filename=filename, path=path, $
                  prefixe=prefixe, suffixe=suffixe)
;
; ici, on va verifier que le fichier est "consistant"
; (meme nombre de delimiteurs par lignes)
;
delim=','
nb_fields=AC_STR_COUNT(a, delim)
if TOTAL(ABS(nb_fields-SHIFT(nb_fields,1)) NE 0) then begin
    print, mess.warning+'Different values of NB_FIELDS encoutered ...'
    STOP
endif else begin
    ;; si on a N champs, on a N-1 separateurs ...
    nb_bolo=nb_fields(0)-1
endelse
;
; on retire la premiere ligne qui contient les noms de chaque champ
;
;tab_lines=tab_lines(1:*)
;nb_bolo=nb_bolo-1
;
if KEYWORD_SET(debug) then stop
if KEYWORD_SET(verbose) then begin
;  print, mess.info+'Total number of bolo read in file', N_ELEMENTS(tab_lines)
endif
;
; template pour la struture bolo
; (attention aux convertions forcees de type (si Int dans la
; declaration, reste ensuite a Int))
;
bolo={name   :'template', $
      Freq   : 111., $       ;; en GHz
      flag   : 1, $          ;; un flag qui dit si le bolo est OK
      Polar  : 0, $          ;; boolean: 0 si flux, 1 si polar
      Rinf   : 100., $       ;; anciennement "r0"
      Ttrans : 10., $         ;; anciennement "Tg"
      Alpha  : 0.5, $        ;; anciennement "m"
      Gbol   : 100.e-12, $   ;; anciennement "g_s0"
      Beta   : 1., $
      c0     : 1.e-12, $
      dcapa  : 1., $
      i_polar: 1.e-10, $
      Tref   : 0.1}   ;; T_bath (Kelvin)

bolo_list=REPLICATE(bolo,nb_bolo)
;
; clefs triviales (communes a touts)
;
clefs=['"R*"','"Tg"','"b"','"G0"','"goes']
nb_clefs=N_ELEMENTS(clefs)
flag_clefs=REPLICATE(0,nb_clefs)
;
; On cherche si chaque clef est bien presente,
; et si oui, on
;
for ii=0, nb_clefs-1 do begin
    quelle_ligne=WHERE(STRPOS(a,clefs[ii]) EQ 0, nbl)
    if (nbl NE 1) then begin
        print, mess.fatal+'Clef was found more than one time ! ', clefs[ii]
    endif else begin
        ligne_split=STRSPLIT(a[quelle_ligne], delim,/extract,/preserve)
        if (ii EQ 0) then bolo_list(*).Rinf=FLOAT(ligne_split(2:*))
        if (ii EQ 1) then bolo_list(*).Ttrans=FLOAT(ligne_split(2:*))
        if (ii EQ 2) then bolo_list(*).Alpha=FLOAT(ligne_split(2:*))
        if (ii EQ 3) then bolo_list(*).Gbol=FLOAT(ligne_split(2:*))
        if (ii EQ 4) then bolo_list(*).flag=FIX(ligne_split(2:*))
        ;;
        ;; on indique dans le flag que la clef a ete traitee
        flag_clefs[ii]=1
    endelse
endfor
;
; on verifie que toutes les clefs triviales prevues ont bien ete trouvees
;
if TOTAL(flag_clefs) NE nb_clefs then begin
    print, mess.warning+'Some Fields have not been found'
    print, 'Key list: .....', clefs
    print, 'Flag list: ....', flag_clefs
    STOP
endif
;
; si on est la, on a suppose que le fichier contient Dark, PSB ou SWB
; on va indique si c'est trouve ou pas (sinon, mefiance, sauf DARK)
;
flag_PSB_or_SWB=0
;
; cas des fichiers SWB
;
if (STRPOS(filename,'PSB') GE 0) then begin
    bolo_list(*).Polar=1
    flag_PSB_or_SWB=1
    ;; on duplique le champ "module"
    quelle_ligne=WHERE(STRPOS(a,'"Module"') EQ 0, nbl)
    ligne_split=STRSPLIT(a[quelle_ligne], delim,/extract,/preserve)
    indices=2*INDGEN(nb_bolo/2+1)
    ligne_split(indices+1)=ligne_split(indices)
    name=ligne_split(2:*)
    ;; on lit le champ "Pinout"
    quelle_ligne=WHERE(STRPOS(a,'"Pinout"') EQ 0, nbl)
    ligne_split=STRSPLIT(a[quelle_ligne], delim,/extract,/preserve)
    ;; on concatene le tout dans le nom du bolo
    bolo_list(*).name=AC_STR_REMOVE(name+'_'+ligne_split(2:*),'"',/all)
endif
;
; cas des fichiers PSB
;
if (STRPOS(filename,'SWB') GE 0) then begin
    bolo_list(*).Polar=0
    flag_PSB_or_SWB=1
    ;; a defaut de clef ... on suppose que c'est tjrs en 2ieme ligne
    quelle_ligne=1
    ligne_split=STRSPLIT(a[quelle_ligne], delim,/extract,/preserve)
    bolo_list(*).name=AC_STR_REMOVE(ligne_split(2:*),'"',/all)
endif
;
; cas des fichiers Dark
;
if (STRPOS(filename,'Dark') GE 0) then begin
    flag_PSB_or_SWB=1
    ;;
    bolo_list(*).Polar=0   ;; a verifier
    bolo_list(*).freq=143. ;; a verifier
    ;; on recupere les noms par la ligne suivante de "Dark"
    quelle_ligne=WHERE(STRPOS(a,'"Dark') EQ 0, nbl)
    quelle_ligne=quelle_ligne+1
    ligne_split=STRSPLIT(a[quelle_ligne], delim,/extract,/preserve)
    bolo_list(*).name=AC_STR_REMOVE(ligne_split(2:*),'"',/all)
endif

;
; En cas de probleme ...
;
if (flag_PSB_or_SWB EQ 0) then begin
    print, mess.warning+'File name is strange (must have PSB or SWB).'
endif
;
if KEYWORD_SET(debug) OR KEYWORD_SET(test) then stop
;
nb_good_bolo=N_ELEMENTS(bolo_list)
if (nb_good_bolo GT 0) then begin
    print, mess.info, nb_good_bolo, ' Bolo are now available.'
endif
;
; on filtre (eventuellement) la liste pour ne garder que les bolo
; - (avant) dont les champs sont complets (toutes les info sont dispo)
; - PFM : qui vont reellement dans le plan focal
;
if KEYWORD_SET(flag) then begin
    print, mess.info+'Bad Bolo are removed from Bolo list.'
    bolo_list=LIST_BOLO_EXTRACT(bolo_list, /flag)
endif
;
nb_good_bolo=N_ELEMENTS(bolo_list)
;
if KEYWORD_SET(verbose) then begin
    print, mess.info+'Total number of GOOD bolo : .....', nb_good_bolo
endif
;
if (nb_good_bolo GT 0) then begin
    print, mess.info, nb_good_bolo, ' Good Bolo are now available.'
endif else begin
    print, mess.warning+'NO good bolo are now available.'
    print, mess.warning+'Please check PATH and Filename.'
endelse
;
if KEYWORD_SET(debug) OR KEYWORD_SET(test) then stop
;
return, bolo_list
;
end
