;
; AC le 07/11/2005 a Paris
;
; "kernel" du calcul:
; Calcule la reponse pour une liste de Frequences et de Courants
; pour un bolo (structure bolo), un niveau de background
; et un niveau d'illumination
;
; ---------------------------------
; Liste des modifications:
;
; AC le 12/12/05: 
; prise en compte 1/ du pre-calcul de la temperature d'equilibre
; pour le courant ET la puissance optique basse et 2/ du temps
; de montee initiale (problematique a plus haute frequence ...)
;
; ---------------------------------
;
function FUNC_BOLO_SPEED, liste_frequencies, liste_currents, liste_params, $
                          time=time, short=short, $
                          test=test, debug=debug, verbose=verbose
;
; eventuelle mesure de la duree du calcul
if KEYWORD_SET(time) then t0=systime(1)
;
; on recopie les parametres selon leurs noms usuels
;
flux_levels=liste_params(0:1)
low_lvl=flux_levels(0)
high_lvl=flux_levels(1)
;
bolo={ name:'Unknown here !', $
       freq:liste_params(2), $
       flag:liste_params(3), $
       polar:liste_params(4), $
       Rinf:liste_params(5), $
       Ttrans:liste_params(6), $
       alpha:liste_params(7), $
       Gbol:liste_params(8), $
       beta:liste_params(9), $
       c0:liste_params(10), $
       Dcapa:liste_params(11)}
;
T_bath=liste_params(12)
;
if KEYWORD_SET(debug) then HELP, /struct, bolo else debug=0
;
nb_freqs=N_ELEMENTS(liste_frequencies)
nb_cur=N_ELEMENTS(liste_currents)
;
; le pas "optimum"
;
delta_t_opti=1.e-4
;
; le tableau des resultats
;
resultat=FLTARR(nb_freqs,nb_cur)
;
; on a un probleme pour le temps de montee du bolo
; on calcule en amont la Temperature d'equilibre
;
Tab_TempInit=FLTARR(nb_cur)
for j=0, nb_cur-1 do begin
    nbp=ROUND(0.2/delta_t_opti)
    op=REPLICATE(low_lvl, nbp)
    ;;op=op+(high_lvl-low_lvl)/2.
    temp=BOLO_DIRECT_MODEL(op, delta_t=delta_t_opti, T_bath=T_bath, $
                           bolo=bolo, cur=liste_currents(j))
    Tab_TempInit(j)=temp(nbp-1)
endfor
if (debug EQ 2) then STOP
;
; Il faut compter de l'ordre de 0.2 sec pour se stabiliser
; si on module aux freq. les plus elevees.
;
duration_up_ref=0.05
;
; les "vrais" calculs
;
for i=0, nb_freqs-1 do begin
    ;;
    nb_samples=128L
    freq=liste_frequencies(i)
    ;; faut-il allonger le temps de calcul ?
    ;; car il faut un pas de calcul de l'ordre de "delta_t_opti" pour que le calcul
    ;; de la reponse du bolo soit fait correctement
    while (freq*nb_samples LE 1./delta_t_opti) do nb_samples=nb_samples*2L
    ;;
    delta_t=1./(freq*nb_samples) 
    indice=1
    ;; calcul de la duree de la FFT
    duration_fft=1./freq
    ;; calcul du nb de points durant la stabilisation
    nbp_up=ROUND(duration_up_ref/delta_t)
    nbp_up=(nbp_up / nb_samples) * nb_samples
    duration_up=nbp_up*delta_t
    ;; on cree le signal creneaux sur la duree qui nous interesse
    duration=duration_up+duration_fft
    nbp=nbp_up+nb_samples
    u=CRENEAUX(nbp=nbp, freq=freq, duration=duration, ishift=nbp_up)
    u=SHIFT(u,nbp/2)
    u=u*(high_lvl-low_lvl)
    op=u+low_lvl
    ;;
    if KEYWORD_SET(verbose) then begin
        print, nb_samples, nbp_up
        print, 'Attention:', freq, nb_samples, nbp, delta_t, duration, freq*duration
    endif
    if (debug EQ 2) then STOP
    ;;
    for j=0, nb_cur-1 do begin
        ;; calcul de la reponse du bolo
        temp=BOLO_DIRECT_MODEL(op, delta_t=delta_t, T_bath=T_bath, $
                               TempInit=Tab_TempInit(j), $
                               bolo=bolo, cur=liste_currents(j))
        ;; extraction du bloc "synchro"
        sub_temp=temp(nbp_up:*)
        ;; conversion Temp --> Volt
        sub_temp=TEMPERATURE2VOLTAGE(sub_temp, liste_currents(j), bolo=bolo)
        ;; calcul du spectre de puissance
        FFT_temp=ABS(FFT(sub_temp))
        ;; extraction de la raie (detection "synchrone")
        resultat(i,j)=FFT_temp(indice)
        ;;
        if (debug EQ 3) and (j EQ 0) then begin
            plot, temp,/ynoz
            ac_lines, Tab_TempInit(j), col=!col.r
            ;;PLOT_POWER_SPECTRUM, sub_temp, duration=duration, central=freq, $
            ;;  freq=MAX([20., freq*2.5]), /nozero, psym=-2
            AC_DELAIS
        endif
    endfor
endfor
;
; eventuel mesure de la duree du calcul
if KEYWORD_SET(time) then begin
    print, 'Computing time in FUNC_BOLO_SPEED2:', systime(1)-t0
endif
;
if KEYWORD_SET(test) then STOP
;
; ne me demandez pas pourquoi il y a ce facteur 1000. !
;
return, TRANSPOSE(resultat)*1000.
;
end
