;
; AC le 07/11/2005 a Paris
;
; "kernel" du calcul:
; Calcule la reponse pour une liste de Frequences et de Courants
; pour un bolo (structure bolo), un niveau de background
; et un niveau d'illumination
;
function FUNC_BOLO_SPEED_REF1, liste_frequencies, liste_currents, liste_params, $
                          short=short, time=time, $
                          test=test, debug=debug, verbose=verbose
;
; eventuelle mesure de la duree du calcul
if KEYWORD_SET(time) then t0=systime(1)
;
; on recopie les parametres selon leurs noms usuels
;
flux_levels=liste_params(0:1)
low_lvl=flux_levels(0)
high_lvl=flux_levels(1)
;
bolo={ name:'Unknown here !', $
       freq:liste_params(2), $
       flag:liste_params(3), $
       polar:liste_params(4), $
       Rinf:liste_params(5), $
       Ttrans:liste_params(6), $
       alpha:liste_params(7), $
       Gbol:liste_params(8), $
       beta:liste_params(9), $
       c0:liste_params(10), $
       Dcapa:liste_params(11)}
;
T_bath=liste_params(12)
;
if KEYWORD_SET(debug) then HELP, /struct, bolo else debug=0
;
; un tableau de resultat
;
nb_freqs=N_ELEMENTS(liste_frequencies)
nb_cur=N_ELEMENTS(liste_currents)
;
; on a un probleme pour le temps de montee du bolo
;
nbp=1.e4
nbp=LONG(nbp)
;
; fenetre utile
;
fenetre=2L^13
;
; 2 tableaux resultat car 2 methodes
;
resultat=FLTARR(nb_freqs,nb_cur)
;
for i=0, nb_freqs-1 do begin
    ;;
    freq=liste_frequencies(i)
    duration=FLOOR(freq)/freq
    ;;
    oversample=1
    if KEYWORD_SET(short) then begin
        nbp=LONG(1e4)
        fenetre=2L^13
        nbp_debut=nbp-fenetre
        if (freq GE 4.) then begin
            oversample=FLOOR(Alog(freq)/Alog(2.))
            fenetre=fenetre/oversample
            duration=duration/oversample
            nbp=nbp_debut+fenetre
            if KEYWORD_SET(verbose) then print, 'OverSample Gain is:', oversample
        endif
    endif
    ;; on cree le signal creneaux sur la duree qui nous interesse
    u=CRENEAUX(nbp=fenetre, freq=freq, duration=duration)
    u=u*(high_lvl-low_lvl)
    ;; on etend ce signal AVANT pour calculer la montee du bolo en temp
    op=REPLICATE(low_lvl, nbp)
    op(nbp-fenetre:*)=op(nbp-fenetre:*)+u
    ;;
    delta_t=duration/fenetre
    indice=ROUND(freq*duration)
    ;;
    if KEYWORD_SET(verbose) then begin
        print, 'Attention:', freq, delta_t, duration, freq*duration
    endif
    if (debug EQ 2) then STOP
    ;;
    ;; obsolete
    ;; op=CREATE_CRENEAUX(nbp, largeur, low=low_lvl, high=high_lvl)
    ;;
    for j=0, nb_cur-1 do begin
        ;; calcul de la reponse du bolo
        temp=BOLO_DIRECT_MODEL(op, delta_t=delta_t, T_bath=T_bath, $
                               bolo=bolo, cur=liste_currents(j))
        ;; extraction du bloc "synchro"
        sub_temp=temp(nbp-fenetre:*)
        ;; conversion Temp --> Volt
        sub_temp=TEMPERATURE2VOLTAGE(sub_temp, liste_currents(j), bolo=bolo)
        ;; calcul du spectre de puissance
        FFT_temp=ABS(FFT(sub_temp))
        ;; extraction de la raie (detection "synchrone")
        resultat(i,j)=FFT_temp(indice)
        ;;
        if (debug EQ 3) and (j EQ 0) then begin
            PLOT_POWER_SPECTRUM, sub_temp, duration=duration, central=freq, $
              freq=MAX([20., freq*1.5]), /nozero, psym=-2
            AC_DELAIS
        endif
    endfor
endfor
; eventuel mesure de la duree du calcul
if KEYWORD_SET(time) then begin
    print, 'Computing time in FUNC_BOLO_SPEED:', systime(1)-t0
endif
;
if KEYWORD_SET(test) then STOP
;
; ne me demandez pas pourquoi il y a ce facteur 1000. !
;
return, TRANSPOSE(resultat)*1000.
;
end
