;
; AC, 22 novembre 2005 a Palaiseu, lumiere d'hiver
;
; relit un fichier de sortie d'un fit et superpose aux donnees brutes 
; du fichier de reponse courant/frequence de modulation
; optique/signal electrique
;
; Attention, ici, les dimensions des 2 jeux de donnees peuvent etre differents
; -------------------------------------------
; -------------------------------------------
;
pro SPEED_DATA_FIT_DISPLAY, fit_filename=fit_filename, postscript=postscript, $
                            full_data=full_data, auto_color=auto_color, $
                            test=test, debug=debug, help=help, verbose=verbose
;
mess=AC_SETMESSAGES('SPEED_DATA_FIT_DISPLAY')
;
if KEYWORD_SET(help) then begin
    print, 'pro SPEED_DATA_FIT_DISPLAY, fit_filename=fit_filename, postscript=postscript, $'
    print, '         test=test, debug=debug, help=help, verbose=verbose'
    return
endif
;
; lecture du fichier de fit
;
OK=AC_SELECTFILE(filename=fit_filename, prefixe='fit_', suffixe='.xdr')
if (OK EQ 1) then begin
    RESTORE, file=fit_filename, /verbose
    print, mess.info+'fit file << '+fit_filename+'>> was succesfully read'
endif else return
;
; lecture des donnees brutes
;
OK=SPEED_DATA_SELECT(filename=speed_name)
if (OK EQ 1) then begin
    SPEED_DATA_READFILE, filename=speed_name, data=data_raw, $
      frequencies=freq_raw, currents=cur_raw
    print, mess.info+'data file <<'+speed_name+'>> was succesfully read'
endif else return
;
; Attention, les donnees brutes peuvent etre de taille superieures
; aux donnees reellement utilisees pour faire l'ajustement
;
if NOT(KEYWORD_SET(full_data)) then begin
    sizes_are_diff=0
    if (N_ELEMENTS(freq_raw) NE N_ELEMENTS(frequencies)) then  sizes_are_diff=1
    if (N_ELEMENTS(cur_raw) NE N_ELEMENTS(currents)) then  sizes_are_diff=1
    if (sizes_are_diff NE 0) then begin
        print, mess.warning+'Data and Fit Freqs. (or Cur.) are differents, consider /full_data keyword'
    endif
    used_freq=frequencies
    used_cur=currents
    used_color=!col.r
endif else begin
    print, mess.info+'Fit calculations are done on data informations (Freq. and Cur.)'
    used_freq=freq_raw
    used_cur=cur_raw
    used_color=!col.g
endelse
;
; comme on n'a sauve que les parametres du FIT , il faut recalculer
; pour afficher les traces ...
;
data_fit=FUNC_BOLO_SPEED(used_freq, used_cur, parms)
;
;
nb_lines=MAX([N_ELEMENTS(used_cur),N_ELEMENTS(cur_raw)])
lines=INDGEN(nb_lines) mod 4
;
; epaisseur du trait
;
lines=REPLICATE(0,nb_lines)
thick=2.
psym=-1
;
; superposition des donnees, du modele avec les valeurs initiales (rouge)
; et du modele valeurs fittees (vert) sur un nombre limite de points
;
if KEYWORD_SET(postscript) then begin
    set_plot, 'ps'
    device, file='speed_bolo'+bolo.name+'.ps', /color, bit=8
endif
;
SPEED_DATA_PLOT, data_raw, freq_raw, cur_raw, title=speed_name, thick=thick, $
  linestyle=lines, psym=psym,/legend
;
lines=REPLICATE(2,nb_lines)
psym=0
;
if NOT(KEYWORD_SET(auto_color)) then begin
    SPEED_DATA_PLOT, data_fit, used_freq, used_cur, /over, thick=thick, $
      col=used_color, linestyle=lines, psym=psym
endif else begin
    SPEED_DATA_PLOT, data_fit, used_freq, used_cur, /over, thick=thick, $
      linestyle=lines, psym=psym
endelse
;
if KEYWORD_SET(postscript) then begin
    device, /close
    set_plot, 'x'
endif
;
if KEYWORD_SET(test) then STOP
;
end
