;
; AC le 05/11/2005 a Palaiseau
; 10 jours de violence dans les Banlieues
;
; Ce programme lit un tableau CSV un peu bancal
; contenant un tableau 2D mixte avec des courants,
; des frequences, et les mesures sur les bolo
;
; ---------------------------------------------
; Liste des modifications
; AC le 18/11/2005:
; -- conversion Courants taille tableau suite ajout donnees MPiat
;    pour les autres bolo (217 et au dessus)
; AC le 21/11/2005
; -- on renomme de READ_DATA_SPEED SPEED_DATA_READFILE
;
; ---------------------------------------------
;
pro SPEED_DATA_READFILE, filename=filename, path=path, $
                data=data, frequencies=frequencies, currents=currents, $
                verbose=verbose, debug=debug, test=test, help=help
;
mess=AC_SETMESSAGES('SPEED_DATA_READFILE')
;
; si le mot-clef /help est active, on retourne l'aide et on sort
;
if KEYWORD_SET(help) then begin
    print, 'pro SPEED_DATA_READFILE, filename=filename, path=path, $'
    print, '         data=data, frequencies=frequencies, currents=currents, $'
    print, '         verbose=verbose,debug=debug, test=test, help=help'
    print, ''
    print, 'This procedure reads an ASCII file containing'
    print, 'experimetal data concerning Speed of Bolo'
    print, 'The output is stored in a list of structure'
    print, ''
    print, 'CALLING SEQUENCE : SPEED_DATA_READFILE, filename="speed.csv", data=data, frequencies=frequencies, currents=currents'
    return
endif
;
; AC 21/05/2005 : we do not manage fully the PATH now
;
if N_ELEMENTS(path) EQ 0 then begin
    path='Data'+!path_sep
    print, mess.info+'Default Path: ./', path
endif
;
if N_ELEMENTS(filename) EQ 0 then begin
    OK=SPEED_DATA_SELECT(filename=filename, path=path)
    if (OK NE 1) then return
    print, mess.info+'Default Speed Data FileName used:', filename
endif
;
full_filename=path+filename
;
; On s'assure que le Fichier est bien present
;
liste_file=AC_FINDFILE(full_filename, count=nb_file)
if (nb_file LT 1) then begin
    print, mess.fatal+'Bolo SPEED FileName <<'+filename+'>>,'
    print, mess.fatal+'in PATH <<'+path+'>> does not exist,'
    print, mess.fatal+'Please re-run this function with another filename.'
    return
endif
;
; Conventions dans le fichier ASCII :
;   - les separateurs sont des ";"
;   - les commentaires sont apres des "#"
;   - la premiere colonne contient le Nom du bolo
;
a=READ_ASCII(full_filename, com='#', del=';', RECORD_START=1)
;
field_name_OK=0
;
if (TAG_NAMES(a) EQ 'FIELD01') then begin
    field_name_OK=1
    tab_out=a.FIELD01
endif
if (TAG_NAMES(a) EQ 'FIELD1') then begin
    field_name_OK=1
    tab_out=a.FIELD1
endif
if (field_name_OK EQ 0) then begin
    print, mess.fatal+'Problem with Field name in Structure'
    STOP
endif
;
nb_freq=(SIZE(tab_out))(2)
nb_cur=(SIZE(tab_out))(1)
nb_cur=nb_cur-1 ; car la 1ere colonne est pour les freqs !
;
print, size(tab_out), nb_freq, nb_cur
;
data=tab_out(1:*,*)
list_freq=REFORM(tab_out(0,*))
tab_cur=FLTARR(nb_cur)
;
; on lit le fichier pour avoir les courants
;
ligne=''
;
GET_LUN, lun
OPENR, lun, full_filename
READF, lun, ligne
READF, lun, ligne  ;; AC 18/11/05 c'est desormais la 2ieme ligne ...
CLOSE, lun
FREE_LUN, lun
;
tab_cur_str=STRSPLIT(ligne,';',/extract)
;
; AC modif le 18/11/2005 suite ajout donnees MPiat
; AC le 18 finalement, pas de modif a cause READ_ASCII
tab_cur=FLOAT(tab_cur_str(1:*))
;
; on verifie qu'il y a bien consistance antre les 2 tableaux
;
new_nb_cur=N_ELEMENTS(tab_cur)
if (new_nb_cur NE nb_cur) then begin
    print, mess.fatal+'Size problem with Currents arrays'
    stop
endif
;
frequencies=list_freq
currents=tab_cur*1e-9
;
; affichage
; Pour le moment, par defaut, on affiche
;
SPEED_DATA_PLOT, data, frequencies, currents, psym=-1, title=filename,/legende
;
if KEYWORD_SET(test) then STOP
;
end
