;
; Creation par AC le 05/04/2004 a Paris
; (le jour de l'arrivee de Boris Fouks )
;
; Description :
; Dans le contexte des simulations de reponse des bolometres de
; Planck, on a besoin de generer des series temporelles d'au moins 4
; types: des lobes (/BEAM), des portes (creneaux, /GATE), des glitches
; (/GLITCH) et des marches (/STEP). Cette procedure permet de generer
; au choix l'un des ces 4 types de signal, avec qlq parametres communs
; (centre, amplitude, speed (du defilement du satelitte), width)
;
; Subtilites:
; - il peut y avoir des unites physiques (cas du lobe gaussien)
;   mais aussi des unites Pixel
;
; Historique des modifications:
;
function CREATE_OPT_SIGNAL, delta_t=delta_t, nbp=nbp, $
                            background=background, amplitude=amplitude, $
                            speed=speed, width=width, $
                            center=center, position=position, $
                            debut=debut, fin=fin, $
                            beam=beam, step=step, gate=gate, glitch=glitch, $
                            display=display, verbose=verbose, $
                            debug=debug, test=test, help=help
;
mess=AC_SETMESSAGES('CREATE_OPT_SIGNAL')
;
;if (N_PARAMS() lt 1) then begin
;    print, mess.fatal+'missing arguments'
;    print, 'out of date'
;    return, -1
;endif
;
if KEYWORD_SET(help) then begin
    print, 'out of date'
    return, -1
endif
;
;-----------------------------------------------------------
; Un et un seul mot-clef dans {beam|gate|glitch|step}
; doit etre actif a la fois. On verifie !
;
;
beam_is_set=0
if KEYWORD_SET(beam) then beam_is_set=1
;
gate_is_set=0
if KEYWORD_SET(gate) then gate_is_set=1
;
glitch_is_set=0
if KEYWORD_SET(glitch) then glitch_is_set=1
;
step_is_set=0
if KEYWORD_SET(step) then step_is_set=1
;
is_set=(beam_is_set+gate_is_set+glitch_is_set+step_is_set)
if (is_set NE 1) then begin
    print, mess.fatal+" Problem with mandatory Keyword {beam|gate|glitch|step}"
    print, mess.fatal+" One and only one Keyword in this list must be set"
    return, -1
endif
;
;-----------------------------------------------------------
; test de consistance pour les autres parametres
; ---------------------------------------------
; meme si on ne peut avoir que des amplitudes POSITIVES,
; on ne genere qu'un WARNING en cas de combinaison de signaux ...
;
if (N_ELEMENTS(amplitude) NE 1) then begin
    print, mess.fatal+"Mandatory Keyword Amplitude is missing."
    return, -1
endif
if (amplitude LE 0) then begin
    print, mess.warning+"unespected NEGATIVE value for Keyword Amplitude ..."
endif
;
; largeur du lobe et vitesse de defilement ...
;
if (N_ELEMENTS(speed) NE 1) then begin
    speed=6.*60.      ;; 6 degres par seconde (en arcmin/s)
endif else begin
    txt="Keyword <<speed>> (in degres/seconde) : "
    if (OUT_OF_RANGE(speed, min=1.,max=10.,txt) NE 0) then begin
        print, mess.fatal+'pb with Speed value.'
        STOP
    endif
endelse
;
if (N_ELEMENTS(width) NE 1) then begin
    width=5.         ;; 5 arc_min
endif else begin
    txt="Keyword <<width>> (in ArcMin) : "
    if (OUT_OF_RANGE(width, min=1.,max=10.,txt) NE 0) then begin
        print, mess.fatal+'pb with Width value.'
        STOP
    endif
endelse
;
; ---------------------- position du signal ---------
;
; pour le moment, l'idee est que le mot-clef Center est prioritaire
; sur Position
;
if KEYWORD_SET(center) then begin
    if (N_ELEMENTS(position) GT 0) then begin
        print, mess.warning+"conflicting keywords Center and Position, using Center"
    endif
    position=nbp/2
endif
if (N_ELEMENTS(position) LE 0) then begin
    print, mess.fatal+"missing keyword Center OR Position"
    return, -1
endif
if (N_ELEMENTS(position) GT 1) then begin
    print, mess.fatal+"size of keyword Position is now limited to 1"
    return, -1
endif
;
if (OUT_OF_RANGE(position, min=0,max=nbp-1,'position') NE 0) then begin
    print, mess.fatal+'Position is out of range'
    return, -1
endif
;
; ---------------------------------------------
; nombre de points et pas d'echantillonnage
;
if (N_ELEMENTS(nbp) NE 1) then begin
    nbp=1e4          ;; soit 1 seconde complete
    delta_t=1e-4  ;; en seconde
endif else begin
    delta_t=1./FLOAT(nbp)
endelse
nbp=LONG(nbp)
;
; -----------------------------------------------
; A partir d'ici, tous les parametres necessaires devraient avoir ete
; actives et testes.
;
if KEYWORD_SET(verbose) then begin
    ;;
    ;; un et un seul est actif !
    if (beam_is_set EQ 1) then print, mess.info+'signal is BEAM'
    if (gate_is_set EQ 1) then print, mess.info+'signal is GATE'
    if (glitch_is_set EQ 1) then print, mess.info+'signal is GLITCH'
    if (step_is_set EQ 1) then print, mess.info+'signal is STEP'
    ;;
    ;; parametres du signal
    print, mess.info+'speed is: .............', speed
    print, mess.info+'amplitude is: .........', amplitude 
    print, mess.info+'center is : ...........', center
    print, mess.info+'position is : .........', position
    print, mess.info+'width is : ............', width
    ;;
    ;; parametres "globaux"
    print, mess.info+'delta_t is: ...........', delta_t
    print, mess.info+'nbp     is: ...........', nbp
    print, mess.info+'background level is: ..', background
    ;;
endif
;
;-----------------------------------------------------------
; debut des calculs du signal
;
; Attention aux 2 cas unites Pixel ou Physique
;
if KEYWORD_SET(beam) then begin
    ;; a ce jour, seul le mode "physique" est actif
    ;;
    at_time=position*delta_t
    signal=HFI_BEAM_GAUSS(nbp, amplitude, width, at_time, $
                          delta_t=delta_t, speed=speed)
endif
;
; les 2 modes devraient etre actifs ...
;
if KEYWORD_SET(gate) then begin
    signal=FLTARR(nbp)
    signal(position-width/2:position+width/2)=amplitude
endif
;;
;; pour ces 2 classes, seulement le mode pixel est actif
;; 
if KEYWORD_SET(glitch) then begin
    signal=FLTARR(nbp)
    signal(position)=amplitude
endif
if KEYWORD_SET(step) then begin
    signal=FLTARR(nbp)
    signal(position:*)=amplitude
endif
;
;-----------------------------------------------------------
;
if (N_ELEMENTS(background) EQ 1) then begin
    signal=signal+background
    if KEYWORD_SET(verbose) then print, mess.info+"background was added."
endif
;
;-----------------------------------------------------------
;
; affichage eventuel
;
if KEYWORD_SET(display) then begin
    WINDOW, /free
    Plot, signal
    AC_DELAIS
    WDELETE, !D.Window
endif
;
if KEYWORD_SET(debug) then begin
    HELP
    STOP
endif
;
if KEYWORD_SET(test) then STOP
;
return, signal
;
end
;
