;
; Creation par AC le 06/02/2004 a Paris LERMA
;
; Using PLANCK_INIT_JUPITER and PLANCK_INIT_FOV,
; this program compute the flux for Jupiter,
; including the nb_modes factor
;
; Historique des modifications:
; AC, le 16/02/04 : - ajout de la valeur du diametre apparent de Jupiter
;                   - fin du codage des calculs
;                   - ajout du mot-clef diametre (au cas ou !)
; AC, le 03/04/04 : - ajout des mots-clefs "largeur_freq" et "optical_eff"
;                   - gestion des limites Min/Max de certains
;                     parametres (diametre, largeur_freq et optical_eff)
;
; Input/Output :
; - this module create with DEFSYSV a !var named : !jupiter_flag_flux
;
;---------------------------------------------
; Ce programme utilise AC_SETMESSAGES et OUT_OF_RANGE
;---------------------------------------------
;
pro JUPITER_T2FLUX, diametre=diametre, $
                    largeur_freq=largeur_freq, optical_eff=optical_eff, $
                    bavard=bavard, test=test, help=help
;
mess=AC_SETMESSAGES('JUPITER_T2FLUX')
;
if KEYWORD_SET(help) then begin
    print, 'pro JUPITER_T2FLUX, diametre=diametre, $'    
    print, '                    largeur_freq=largeur_freq, optical_eff=optical_eff, $'
    print, '                    bavard=bavard, test=test, help=help'
    print, ' '
    print, 'This program computes, for each Planck HFI bolometer,'
    print, 'the observed Flux of Jupiter using Jupiter Brightness Temperature'
    print, 'and diameter, given the FoV and the nb_modes factor.'
    return
endif
;
;--------------------------------------------- 
;
; Au cas ou ces modules n'ont pas ete initilises, on le fait !
;
DEFSYSV, '!jupiter_flag_init', exist=flag_init
if (flag_init NE 1) then begin
    print, mess.warning+'PLANCK_INIT_JUPITER was not run before'
    PLANCK_INIT_JUPITER
endif
DEFSYSV, '!fov_flag_init', exist=flag_init
if (flag_init NE 1) then begin
    print, mess.warning+'PLANCK_INIT_FOV was not run before'
    PLANCK_INIT_FOV
endif
;
; 3 param a gerer (optical_eff, diametre, largeur_freq)
;---------------------------------------------
;
k_bolztam=1.38e-23 ;(J/K)
;
; Efficacite optique (entre 1/3 et 1/2)
;
if (N_ELEMENTS(optical_eff) NE 1) then begin
    optical_eff=0.36   ; JML 0.5
endif
if OUT_OF_RANGE(optical_eff, min=0.3, max=0.5, mess.fatal+" [optical_eff] ") then STOP
print, mess.info+"used value for keyword OPTICAL_EFF is", optical_eff
;
; Largeur en frequence du "canal"
; (pour un bolo, pas de pb pour avoir 1/3 !)
;
if (N_ELEMENTS(largeur_freq) NE 1) then begin
    largeur_freq=0.333          ; MPiat 0.3
endif
if OUT_OF_RANGE(largeur_freq, min=0.3, max=0.5, mess.fatal+" [largeur_freq] ") then STOP
print, mess.info+"used value for keyword LARGEUR_FREQ", largeur_freq
; 
; diametre de jupiter (Units : ArcSec)
; sous toute reserve,
; valeur issue de http://www.arnaudxx.net/jupiter.php
; diametre=44.11 ; (diametre de jupiter) Units : ArcSec
;
if (N_ELEMENTS(diametre) NE 1) then begin
    diametre=38.6   ; Units : ArcSec
endif
if OUT_OF_RANGE(diametre, min=30, max=50, mess.fatal+" [diametre] ") then STOP
print, mess.info+"About Jupiter Diameter (Units : ArcSec)"
print, mess.info+"used value for keyword DIAMETRE", diametre
;
diametre=diametre/60. ; on passe en ArcMin
;
; -----------------------------------------------------
; fin de l'initilisation eventuelle des differents parametres,
; debut des calculs !
;
ratio=2.*optical_eff*k_bolztam*largeur_freq
;
nb_modes={f100 : 1., $
          f143 : 1., $
          f217 : 1., $
          f353 : 1., $
          f545 : 6.5, $
          f857 : 16. }
andreas_flag=0
if (andreas_flag EQ 1) then begin
    nb_modes={f100 : 1., $
              f143 : 1., $
              f217 : 1., $
              f353 : 2., $
              f545 : 4.8, $
              f857 : 11.8 }
    print, mess.info+"Andrea's values for Mode numbers"
endif


;
freq={f100 : 100.e9, $
      f143 : 143.e9, $
      f217 : 217.e9, $
      f353 : 353.e9, $
      f545 : 545.e9, $
      f857 : 857.e9}
;
; initialisation de la structure
;
flux={f100 : 1., $
      f143 : 1., $
      f217 : 1., $
      f353 : 1., $
      f545 : 1., $
      f857 : 1. }
;
; modif de la structure
;
flux.f100=freq.f100*ratio*nb_modes.f100*!jupiter_temp.f100*(diametre/!fov.f100)^2
flux.f143=freq.f143*ratio*nb_modes.f143*!jupiter_temp.f143*(diametre/!fov.f143)^2
flux.f217=freq.f217*ratio*nb_modes.f217*!jupiter_temp.f217*(diametre/!fov.f217)^2
flux.f353=freq.f353*ratio*nb_modes.f353*!jupiter_temp.f353*(diametre/!fov.f353)^2
flux.f545=freq.f545*ratio*nb_modes.f545*!jupiter_temp.f545*(diametre/!fov.f545)^2
flux.f857=freq.f857*ratio*nb_modes.f857*!jupiter_temp.f857*(diametre/!fov.f857)^2
;
; on le passe en variable systeme
;
DEFSYSV, '!jupiter_flux', exist=flag_init
if (flag_init EQ 1) then begin
    print, mess.warning+'!jupiter_flux was previously initilized ...'
    !jupiter_flux=flux
endif else begin
    DEFSYSV, '!jupiter_flux', flux
    print, mess.info+"Flux of Jupiter !Jupiter_flux has been computed" 
endelse
;
if KEYWORD_SET(bavard) then begin
    print, mess.info,'content of Structure !jupiter_flux is:'
    HELP,/structure, !jupiter_flux
endif
if KEYWORD_SET(test) then STOP
;
; une fois que tout s'est bien passe, on active le flag
; indiquant qu'on est bien passe ici !
; comme "diametre" peut changer, on ne teste pas si on est deja passe
;
Read_only=1
DEFSYSV, '!jupiter_flag_flux', 1, Read_only
;
end
